print.roc <- function(roc.obj, digits=max(options()$digits - 4,3),print.=T)
  {

   if(!inherits(roc.obj,'roc'))
        stop("Not legitimate roc object")
   savedig <- options(digits=digits)
   on.exit(options(savedig))

   markers <- roc.obj$x
   status <- roc.obj$y
   marker.names <- attr(roc.obj$terms,'term.labels')
   x1 <- (markers[status==1,])
   x0 <- (markers[status==0,])
   num1 <- sum(status)
   num0 <- length(status) - sum(status)
   nmark <- ncol(markers)
   eps <- 10^(-16)

   ### creates num0 by num1 by nmark array
   if(nmark==1){
      diffarray <- outer(x1,rep(1,num0)) - 
                   outer(rep(1,num1),x0)
      countmat <- sum(diffarray>eps) + 0.5*sum(abs(diffarray)<=eps)
     }
   if(nmark>1) {
      diffarray <- aperm(outer(x1,rep(1,num0)),c(3,1,2)) - 
                   aperm(outer(rep(1,num1),x0),c(2,1,3))
      countmat <- apply(diffarray>eps,3,sum) + 
                  0.5*apply(abs(diffarray)<=eps,3,sum)
    }

   mean2 <- function(x) mean(x,na.rm=T)
   sd2   <- function(x) sqrt(var(x[!is.na(x)]))
   area <- countmat/(num1*num0)
   if(sum(area<.5)> 0 & sum(area<.5)< length(area)) 
     cat("Caution: AUC for all markers are printed to be > .5 but high\n", 
	 "        values do not mean the same thing for all markers\n\n") 
   area[area<.5] <- (1-countmat/(num1*num0))[area<.5]

   x0 <- matrix(x0,nrow=num0,ncol=nmark)
   x1 <- matrix(x1,nrow=num1,ncol=nmark)

   output <- matrix(NA,nrow=nmark,ncol=9)
   output[,1] <- signif(area,digits-1)
   output[,2] <- apply(!is.na(x0),2,sum)
   output[,3] <- signif(apply(x0,2,mean2),digits)
   output[,4] <- signif(apply(x0,2,sd2),digits)
   output[,5] <- signif(output[,4]/output[,3],digits-1)

   output[,6] <- apply(!is.na(x1),2,sum)
   output[,7] <- signif(apply(x1,2,mean2),digits)
   output[,8] <- signif(apply(x1,2,sd2),digits)
   output[,9] <- signif(output[,8]/output[,7],digits-1)


   dimnames(output) <- list(marker.names,c('ROC AUC',' Cont(0): n','mean',
                            'std.dev','cv',' Case(1): n','mean','std.dev','cv'))
   if(print.) { if(!is.null(cl <- roc.obj$call)) {
	          cat("Call:\n")
	          dput(cl)
	          }
               cat('\nMarkers:\n')
               print(output)
               cat('\n\n') 
	     }
   
   invisible(output)
 }


