summary.roc <- function(roc.obj, pairs=F, conf.level = 0.95,
                        se.method=c('delong','bootstrap'),
			nboot=100, stratboot=F)

  {

rocboot <- function(roc.obj=roc.obj,nboot=nboot,stratboot=stratboot) {

   markers <- roc.obj$x
   status <- roc.obj$y
   yname <- attr(roc.obj$terms,'variables')[[1]]
   status <- matrix(roc.obj$y)
   dimnames(status) <- list(NULL,yname)
   dimnames(markers) <- list(NULL,attr(roc.obj$terms,"term.labels"))
   marker.names <- attr(roc.obj$terms,'term.labels')
   x1 <- (markers[status==1,])
   x0 <- (markers[status==0,])
   num1 <- sum(status)
   num0 <- length(status) - sum(status)
   nmark <- ncol(markers)
   eps <- 10^(-16)
   nobs <- nrow(markers)


### First select the observation numbers from the dataset cbind(y,x) 
### if stratboot=F

   if(!stratboot) boot <- matrix(sample(1:nobs, size=nobs*nboot, replace=T),
		                 nrow=nobs,ncol=nboot)

   ### If stratboot=T then do the following
   if(stratboot) {

     boot1 <- matrix(sample(1:num0, size=num0*nboot, replace=T),
	  	     nrow=num0,ncol=nboot)
     boot2 <- matrix(sample((num0+1):(num0+num1),size=num1*nboot,replace=T),
		     nrow=num1, ncol=nboot)
     boot <- rbind(boot1,boot2)
   }


### Call roc nboot times and store the results,
### the 'area' under the curve, then calculate 'S' (variance-covariance matrix)

   bootarea <- matrix(NA,nrow=nboot,ncol=nmark)

   for(i in 1:nboot) {
      tmpdata <- data.frame(cbind(status,markers)[boot[,i],])
      tmpfit <- roc(roc.obj$formula,data=tmpdata)
      bootarea[i,] <- print(tmpfit,print=F)[,1]
		   }

   S <- var(bootarea)
   return(S)	       
	     }



## test call is summary(fit,se.method='boot',nboot=2)


   if(!inherits(roc.obj,'roc'))
        stop("Not legitimate roc object")

   se.method <- match.arg(se.method)


   if(se.method=='bootstrap')
        cat('The bootstrap may take awhile ...\n')

   markers <- roc.obj$x
   status <- roc.obj$y
   marker.names <- attr(roc.obj$terms,'term.labels')
   x1 <- markers[status==1,]
   x0 <- markers[status==0,]
   num1 <- sum(status)
   num0 <- length(status) - sum(status)
   nmark <- ncol(markers)
   nobs <- nrow(markers)
   eps <- 10^(-16)
   outmat <- matrix(0,ncol=6,nrow=nmark)


    ### creates num0 by num1 by nmark array
    if(nmark==1){
       diffarray <- outer(x1,rep(1,num0)) - 
                    outer(rep(1,num1),x0)
       countmat <- sum(diffarray>eps) + 0.5*sum(abs(diffarray)<=eps)
      }
    if(nmark>1) {
       diffarray <- aperm(outer(x1,rep(1,num0)),c(1,3,2)) - 
                    aperm(outer(rep(1,num1),x0),c(1,2,3))
       countmat <- apply(diffarray>eps,3,sum) + 
                   0.5*apply(abs(diffarray)<=eps,3,sum)
     }
    area <- matrix(countmat/(num1*num0),nrow=nmark)

    if(sum(area<.5)> 0 & sum(area<.5)< length(area)) 
     cat("Caution: AUC for all markers are printed to be > .5 but high\n", 
	 "        values do not mean the same thing for all markers\n\n") 

    area[area<.5] <- (1-countmat/(num1*num0))[area<.5]

   if(se.method=='delong') {
     #Calculate the S matrix
     countmtx <- (sign(diffarray)+1)/2

     if(nmark==1){
       V.x1 <- matrix(apply(countmtx,1,sum)/num0,ncol=nmark)
       V.x0 <- matrix(apply(countmtx,2,sum)/num1,ncol=nmark)
        }
     if(nmark>1){
       V.x1 <- matrix(apply(countmtx,c(1,3),sum)/num0,ncol=nmark)
       V.x0 <- matrix(apply(countmtx,c(2,3),sum)/num1,ncol=nmark)
        }

     S.1 <- var(V.x1)
     S.0 <- var(V.x0)
     S <- (S.1/num1) + (S.0/num0)
     }

   if(se.method=='bootstrap') 
      S <- rocboot(roc.obj,nboot,stratboot)

   #Calculate the standard error of the area under each curve

   var.area <- diag(S)
   se.area <- sqrt(var.area)

   test.stat <- (area-0.5)/se.area
   p.value <- (1-pnorm(abs(test.stat)))
   z.value <- qnorm(1-((1-conf.level)/2))

   outmat[,1] <- round(area,4)
   outmat[,2] <- round(se.area,4)
   outmat[,3] <- round(test.stat,2)
   outmat[,4] <- round(p.value,4)
   outmat[,5] <- round(area-z.value*se.area,4)
   outmat[,6] <- round(area+z.value*se.area,4)

   dimnames(outmat) <- list(marker.names,
                       c(' ROC AUC',' Std. Error',' z value',' Pr(>z)',
                       paste(' lower.',round(100*(conf.level),2),sep=''),
	               paste(' upper.',round(100*(conf.level),2),sep='')))

   # perform overall test of equality of areas: 
   #  H0:  theta1 = theta2 = theta3 = ...

   if(nmark > 1) {

     L1 <- rep(1,nmark-1)
     L2 <- matrix(0,nmark-1,nmark-1)
     L.equality <- cbind(L1,L2)
     for(i in 1:(nmark-1))  {L.equality[i,i+1] <- (-1)}
     Ltheta <- (L.equality %*% area)

     overall.invLSL <- solve(L.equality %*% S %*% t(L.equality)) 
     overall.test <- as.numeric(t(Ltheta) %*% overall.invLSL %*% (Ltheta))
     overall.pvalue <- (1-(pchisq(overall.test,nmark-1)))
   }
  if(nmark == 1){
     overall.test <- NULL
     overall.pvalue <- NULL
   }

  if(pairs) {
    if(!nmark>1)
        stop("\n    Only 1 marker so can't do pairwise comparisons")
   cat("\n2-sided test\nPairwise Comparisons:\n")

   outmat2 <- matrix(0,nrow=(nmark*(nmark-1)/2), ncol= 6)
   outname <- matrix(NA,nrow=(nmark*(nmark-1)/2), ncol= 1)
    #find pairwise confidence intervals
   counter <- 1
   for(a in 1:(nmark-1)){
       for(b in (a+1):nmark){
           L<-rep(0,nmark)
           L[a]<-(-1)
           L[b]<-1
           L<-matrix(L,nrow=1)
           contr.mean<-as.numeric(L %*% area)
           contr.se<-sqrt(as.numeric(L %*% S %*% (t(L))))
           z.value<-qnorm(1-((1-conf.level)/2))
           test.stat<-contr.mean/contr.se
           p.value2<-2*(1-pnorm(abs(test.stat)))
           upper.ci<-contr.mean + z.value*contr.se
           lower.ci<-contr.mean - z.value*contr.se
	   outname[counter] <- paste(marker.names[b],'-',marker.names[a])
           outmat2[counter,1] <- round(area[b],4)
	   outmat2[counter,2] <- round(area[a],4)
	   outmat2[counter,3] <- round(contr.mean,4)
           outmat2[counter,4] <- round(p.value2,4)
	   outmat2[counter,5] <- round(lower.ci,3)
	   outmat2[counter,6] <- round(upper.ci,3)
	   counter <- counter+1
           }
        }
        dimnames(outmat2) <- list(outname, 
          c(' Area.2',' Area.1',' Area Diff',' P-value ',
            paste(' lower.',round(100*(conf.level),2),sep=''),
	    paste(' upper.',round(100*(conf.level),2),sep='')))
    }

    dimnames(S) <- list(marker.names,marker.names)
    dimnames(area) <- list(marker.names,'ROC AUC')
    p.value <- matrix(p.value)
    dimnames(p.value) <- list(marker.names,'Pr(>|z|)')
    se.area <- matrix(se.area)
    dimnames(p.value) <- list(marker.names,'Std. Err')

    if(!pairs) outmat2 <- NULL
    results <- list(call=roc.obj$call, cov.area = S, area=area, 
		    se.area = se.area, p.value=p.value,
		    pairs.matrix=outmat2, results=outmat, pairs=pairs,
		    se.method=se.method, n.markers=nmark,
		    overall.test=overall.test, overall.pvalue=overall.pvalue)

    class(results) <- 'summary.roc'
    return(results)
   
 }


